/************************************************************************
 *                                                                      *
 *  FreeSynd - a remake of the classic Bullfrog game "Syndicate".       *
 *                                                                      *
 *   Copyright (C) 2005  Stuart Binge  <skbinge@gmail.com>              *
 *   Copyright (C) 2005  Joost Peters  <joostp@users.sourceforge.net>   *
 *   Copyright (C) 2006  Trent Waddington <qg@biodome.org>              *
 *   Copyright (C) 2006  Tarjei Knapstad <tarjei.knapstad@gmail.com>    *
 *                                                                      *
 *    This program is free software;  you can redistribute it and / or  *
 *  modify it  under the  terms of the  GNU General  Public License as  *
 *  published by the Free Software Foundation; either version 2 of the  *
 *  License, or (at your option) any later version.                     *
 *                                                                      *
 *    This program is  distributed in the hope that it will be useful,  *
 *  but WITHOUT  ANY WARRANTY;  without even  the implied  warranty of  *
 *  MERCHANTABILITY  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  *
 *  General Public License for more details.                            *
 *                                                                      *
 *    You can view the GNU  General Public License, online, at the GNU  *
 *  project's  web  site;  see <http://www.gnu.org/licenses/gpl.html>.  *
 *  The full text of the license is also included in the file COPYING.  *
 *                                                                      *
 ************************************************************************/
#ifndef SOUNDMANAGER_H
#define SOUNDMANAGER_H

#include "common.h"
#include <vector>

// These enum values match the indices in the vector containing the samples
// so don't mess up the order in which they are in.
enum InGameSample
  {
    SHOTGUN = 0,
    GUN,
    LASER,
    FLAME,
    FLAMING_DEATH,
    GLASS_BREAKING,
    EXPLOSION,
    UNRECOGNIZED_SHOT_1,
    UNRECOGNIZED_SHOT_2,
    MINIGUN,
    UNRECOGNIZED_1,
    UNRECOGNIZED_2,
    UNRECOGNIZED_3,
    UNRECOGNIZED_4,
    SPEECH_MISSION_COMPLETED,
    SPEECH_MISSION_FAILED,
    DOOR,
    UNRECOGNIZED_5,
    UNRECOGNIZED_6,
    UNRECOGNIZED_7,
    UNRECOGNIZED_8,
    FIREWORKS,
    SPEECH_NO,
    UNRECOGNIZED_9,
    UNRECOGNIZED_10,
    APPLAUSE,
    APPLAUSE_ZOOM,
    FIREWORKS_APPLAUSE,
    EXPLOSION_BIG,
    UNRECOGNIZED_11,
    UNRECOGNIZED_12,
  };

class Sound;

class SoundManager {
public:
    enum SampleSet
      {
	SAMPLES_INTRO,
	SAMPLES_GAME
      };

    SoundManager();
    ~SoundManager();

    Sound *sound(int n);
    Sound *sound(InGameSample sample);
    void loadSounds(SampleSet set);
    
protected:
    bool loadSounds(uint8 * tabData, int tabSize, 
		    uint8 * soundData);

    const int tabentry_startoffset_;
    const int tabentry_offset_;
    std::vector<Sound*> sounds_;
};

#endif

