/************************************************************************
 *                                                                      *
 *  FreeSynd - a remake of the classic Bullfrog game "Syndicate".       *
 *                                                                      *
 *   Copyright (C) 2005  Stuart Binge  <skbinge@gmail.com>              *
 *   Copyright (C) 2005  Joost Peters  <joostp@users.sourceforge.net>   *
 *   Copyright (C) 2006  Trent Waddington <qg@biodome.org>              *
 *   Copyright (C) 2006  Tarjei Knapstad <tarjei.knapstad@gmail.com>    *
 *                                                                      *
 *    This program is free software;  you can redistribute it and / or  *
 *  modify it  under the  terms of the  GNU General  Public License as  *
 *  published by the Free Software Foundation; either version 2 of the  *
 *  License, or (at your option) any later version.                     *
 *                                                                      *
 *    This program is  distributed in the hope that it will be useful,  *
 *  but WITHOUT  ANY WARRANTY;  without even  the implied  warranty of  *
 *  MERCHANTABILITY  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  *
 *  General Public License for more details.                            *
 *                                                                      *
 *    You can view the GNU  General Public License, online, at the GNU  *
 *  project's  web  site;  see <http://www.gnu.org/licenses/gpl.html>.  *
 *  The full text of the license is also included in the file COPYING.  *
 *                                                                      *
 ************************************************************************/
#ifndef XMIDI_H
#define XMIDI_H

#include "common.h"
#include <vector>

class XMidi {
public:
    struct Midi
    {
      uint8* data_;
      int size_;
    };

    // Reads an .xmi file and returns a vector with individual MIDI tracks
    std::vector<Midi> convertXMidi(uint8* buf, int size);

protected:
    // The code below and corresponding implementeation is taken from the
    // The System Shock Hack Project (BSD license), 
    // http://tsshp.sourceforge.net/, Copyright (c) 2001 Ryan Nunn
    // Function names have been changed to protect the innocent (and match our
    // coding standards...)
    struct midi_event
    {
      int              time;
      uint8            status;
      
      uint8            data[2];
      
      uint32           len;
      uint8            *stream;
      
      midi_event       *next;
    };

    struct XMidiFile
    {
      uint16          tracks;
      
      midi_event      **events;
      signed short    *timing;
      
      midi_event      *list;
      midi_event      *current;
      
    };

    /*
     * Loads an XMidi file.
     * Returns NULL if failed
     */
    XMidiFile *loadXMidi(uint8* buf, int size);

    /* Frees an XMidi returned by XMidi_Load */
    void freeXMidi(XMidiFile *xmidi);
    
    /*
     * Creates a buffer that contains a MIDI File.
     * Buffer size is returned.
     * If size is 0, an error was encountered
     */
    int retrieveMidi(XMidiFile *xmidi, uint32 track, uint8 **buffer);
 
private:
    void deleteEventList(midi_event *mlist);
    midi_event *allocateEvent(int time);
    void createNewEvent(XMidiFile *xmidi, int time);
    int getVLQ(const unsigned char *stream, uint32 *quant);
    int getVLQ2(const unsigned char *stream, uint32 *quant);
    int putVLQ(unsigned char *stream, uint32 value);
    int convertEvent(XMidiFile *xmidi, const int time, 
			   const unsigned char status, 
			   const unsigned char *stream, const int size);
    int convertSystemMessage(XMidiFile *xmidi, const int time, 
				   const unsigned char *stream);
    int readEventList(XMidiFile *xmidi, const unsigned char *stream);
    int extractTracks(XMidiFile *xmidi, const unsigned char *stream, 
			    const uint32 size);
    int readFile (XMidiFile *xmidi, const unsigned char *stream);
    uint32 convertListToMTrk (unsigned char *buf, midi_event *mlist);
};

#endif







